"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const npmlog_1 = __importDefault(require("npmlog"));
const chalk_1 = __importDefault(require("chalk"));
const LogLevel_1 = require("../LogLevel");
const formatDuration_1 = require("./formatDuration");
const taskId_1 = require("../../task/taskId");
const maxLengths = {
    pkg: 0,
    task: 0,
};
const colors = {
    info: chalk_1.default.white,
    verbose: chalk_1.default.gray,
    warn: chalk_1.default.white,
    error: chalk_1.default.white,
    task: chalk_1.default.cyan,
    pkg: chalk_1.default.magenta,
};
function getTaskLogPrefix(pkg, task) {
    return `${colors.pkg(pkg.padStart(maxLengths.pkg))} ${colors.task(task.padStart(maxLengths.task))}`;
}
function normalize(prefixOrMessage, message) {
    if (typeof message === "string") {
        const prefix = prefixOrMessage;
        return { prefix, message };
    }
    else {
        const prefix = "";
        const message = prefixOrMessage;
        return { prefix, message };
    }
}
class NpmLogReporter {
    constructor(options) {
        this.options = options;
        this.groupedEntries = new Map();
        options.logLevel = options.logLevel || LogLevel_1.LogLevel.info;
        npmlog_1.default.level = LogLevel_1.LogLevel[options.logLevel];
    }
    log(entry) {
        if (this.options.logLevel >= entry.level) {
            const isTaskLogEntry = entry.data && entry.data.package && entry.data.task;
            if (isTaskLogEntry && !this.options.grouped) {
                return this.logTaskEntry(entry.data.package, entry.data.task, entry);
            }
            else if (isTaskLogEntry && this.options.grouped) {
                return this.logTaskEntryInGroup(entry.data.package, entry.data.task, entry);
            }
            else {
                return this.logGenericEntry(entry);
            }
        }
    }
    logGenericEntry(entry) {
        const normalizedArgs = normalize(entry.msg);
        const logFn = npmlog_1.default[LogLevel_1.LogLevel[entry.level]];
        const colorFn = colors[LogLevel_1.LogLevel[entry.level]];
        return logFn(normalizedArgs.prefix, colorFn(normalizedArgs.message));
    }
    logTaskEntry(pkg, task, entry) {
        const normalizedArgs = this.options.grouped
            ? normalize(entry.msg)
            : normalize(getTaskLogPrefix(pkg, task), entry.msg);
        const logFn = npmlog_1.default[LogLevel_1.LogLevel[entry.level]];
        const colorFn = colors[LogLevel_1.LogLevel[entry.level]];
        if (entry.data && entry.data.status) {
            const pkgTask = this.options.grouped
                ? `${chalk_1.default.magenta(pkg)} ${chalk_1.default.cyan(task)}`
                : "";
            switch (entry.data.status) {
                case "started":
                    return logFn(normalizedArgs.prefix, colorFn(`▶️ start ${pkgTask}`));
                case "completed":
                    return logFn(normalizedArgs.prefix, colorFn(`✔️ done ${pkgTask} - ${formatDuration_1.formatDuration(entry.data.duration)}`));
                case "failed":
                    return logFn(normalizedArgs.prefix, colorFn(`❌ fail ${pkgTask}`));
                case "skipped":
                    return logFn(normalizedArgs.prefix, colorFn(`⏭️ skip ${pkgTask}`));
            }
        }
        else {
            return logFn(normalizedArgs.prefix, colorFn("|  " + normalizedArgs.message));
        }
    }
    logTaskEntryInGroup(pkg, task, logEntry) {
        var _a, _b, _c;
        const taskId = taskId_1.getTaskId(pkg, task);
        this.groupedEntries.set(taskId, this.groupedEntries.get(taskId) || []);
        (_a = this.groupedEntries.get(taskId)) === null || _a === void 0 ? void 0 : _a.push(logEntry);
        if (logEntry.data &&
            (logEntry.data.status === "completed" ||
                logEntry.data.status === "failed" ||
                logEntry.data.status === "skipped")) {
            const entries = this.groupedEntries.get(taskId);
            for (const entry of entries) {
                this.logTaskEntry((_b = entry.data) === null || _b === void 0 ? void 0 : _b.package, (_c = entry.data) === null || _c === void 0 ? void 0 : _c.task, entry);
            }
            this.hr();
        }
    }
    hr() {
        npmlog_1.default.info("", "----------------------------------------------");
    }
    summarize(context) {
        var _a;
        const { measures, tasks } = context;
        const { hr } = this;
        const statusColorFn = {
            completed: chalk_1.default.greenBright,
            failed: chalk_1.default.redBright,
            skipped: chalk_1.default.gray,
        };
        npmlog_1.default.info("", chalk_1.default.cyanBright(`🏗 Summary\n`));
        if (measures.failedTask) {
            const { pkg, task } = measures.failedTask;
            const taskId = taskId_1.getTaskId(pkg, task);
            const taskLogs = (_a = tasks.get(taskId)) === null || _a === void 0 ? void 0 : _a.logger.getLogs();
            npmlog_1.default.error("", `ERROR DETECTED IN ${pkg} ${task}`);
            if (taskLogs) {
                npmlog_1.default.error("", taskLogs === null || taskLogs === void 0 ? void 0 : taskLogs.map((entry) => entry.msg).join("\n"));
            }
            hr();
        }
        if (tasks.size > 0) {
            for (const npmScriptTask of tasks.values()) {
                const colorFn = statusColorFn[npmScriptTask.status];
                npmlog_1.default.info("", getTaskLogPrefix(npmScriptTask.info.name, npmScriptTask.task), colorFn(`${npmScriptTask.status}, took ${formatDuration_1.formatDuration(formatDuration_1.hrToSeconds(npmScriptTask.duration))}`));
            }
        }
        else {
            npmlog_1.default.info("", "Nothing has been run.");
        }
        hr();
        npmlog_1.default.info("", `Took a total of ${formatDuration_1.formatDuration(formatDuration_1.hrToSeconds(measures.duration))} to complete`);
    }
}
exports.NpmLogReporter = NpmLogReporter;
